#ifndef LHAPDF_H
#define LHAPDF_H

#include <string>
#include <vector>
#include "LHAPDFfw.h"

/**
 * @mainpage LHAPDFWrap: a C++ wrapper around the LHAPDF package
 *
 * @section intro Introduction
 * The LHAPDFWrap library provides a set of C++ wrapper functions 
 * for the Fortran LHAPDF library. New users should browse this
 * documentation and take a look at the @c TestCCWrapper.cc
 * source file, which is a good example of how the wrapper library 
 * can be used.
 *
 * @section changes Recent changes
 * The LHAPDF wrapper has been improved in several ways for 
 * the LHAPDF v5.4 release:
 *
 * @li String passing to the Fortran functions from C++ now 
 *  correctly passes the hidden length argument, fixing problems
 *  on 64 bit systems;
 * @li The @c LHAPDFWrap class has been deprecated in favour of 
 *  a set of wrapper functions in the @c LHAPDF namespace. The
 *  class interface was misleading, since no persistent state
 *  was involved and two class instances would not have been 
 *  independent;
 * @li Proper C++ @c std::string arguments can now be used for 
 *  set names: @c char* string arguments can still be passed, due 
 *  to implicit conversion via the string(char*) constructor.
 *
 * @section credits Credits
 *
 * @li Originally by Stefan Gieseke.
 * @li Adapted for LHAPDFv4 by Mike Whalley.
 * @li Adapted for LHAPDFv5 by Craig Group/Mike Whalley.
 * @li LHAPDF v5.4: Fortran portability, tidying and conversion
 *  to namespaced functions by Andy Buckley.
 *
 * @example ../examples/CCTest1.cc
 * This is an example of a program using the recommended C++ 
 * interface to LHAPDF.
 *
 * @example ../examples/CCTest2.cc
 * An example of a program using the C++ interface to LHAPDF to 
 * calculate PDF errors.
 */


/// Namespace containing all the LHAPDF wrapper functions.
namespace LHAPDF {

  /// @brief Enum of flavours which map to LHAPDF integer codes.
  /// Useful for improving readability of client code. Note that these codes
  /// can't be used to access elements of returned @c vector<double>, which
  /// don't use the LHAPDF scheme (they use "LHAPDF code + 6").
  enum Flavour {
    TBAR= -6, BBAR = -5, CBAR = -4, SBAR = -3, UBAR = -2, DBAR = -1,
    GLUON = 0,
    DOWN = 1, UP = 2, STRANGE = 3, CHARM = 4, BOTTOM = 5, TOP= 6,
    PHOTON = 7
  };

  /// @brief Distinction between evolution or interpolation PDF sets.
  /// Enum to choose whether evolution (i.e. @c LHpdf data file) or
  /// interpolation (i.e. @c LHgrid data file) is used.
  enum SetType {
    EVOLVE = 0, LHPDF = 0, 
    INTERPOLATE = 1, LHGRID = 1
  };

  /// Level of noisiness.
  enum Verbosity { SILENT=0, LOWKEY=1, DEFAULT=2 };


  /// @name Single-set functions
  /// LHAPDF functions for operating on a single PDF set. If you need to use
  /// more than one set simultaneously, use the "M-variant" multi-set functions.
  //@{

  /// The PDF set by file path, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSet(const std::string& path);

  /// The PDF set by name and type, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSetByName(const std::string& name, SetType type);

  /// The PDF set by filename, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSetByName(const std::string& filename);

  /// The choice of PDF subset out of one distribution.
  void initPDF(int memset);

  /// Convenient initializer with PDF set @a name, set type @a type and subset @a memset.
  /// @param name The name of the desired set.
  /// @param type The type of PDF set (grid or data) by enum.
  /// @param memset PDF number within set @a name.
  /// Equivalent to @c initPDFSetByName + @c initPDF.
  void initPDFByName(const std::string& name, SetType type, int memset);

  /// Convenient initializer with PDF set @a filename and subset @a memset.
  /// @param filename The name of the grid or data file of the desired set.
  /// @param memset PDF number within set @a name.
  /// Equivalent to @c initPDFSetByName + @c initPDF.
  void initPDFByName(const std::string& filename, int memset);

  /// Prints a brief description of the current PDF set to stdout.
  void getDescription();

  /// Nucleon PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  std::vector<double> xfx(double x, double Q);

  /// Nucleon PDF: fills primitive 13 element array pointed at by @a results with
  /// \f$ x f(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$;
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  void xfx(double x, double Q, double* results);

  /// Nucleon PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  double xfx(double x, double Q, int fl);

  /// Photon PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a P2 and @a ip params.
  std::vector<double> xfxp(double x, double Q, double P2, int ip);

  /// Photon PDF: fills primitive 13 element array pointed at by @a results with
  /// \f$ x f(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$;
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a P2 and @a ip params.
  void xfxp(double x, double Q, double P2, int ip, double* results);

  /// Photon PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a P2 and @a ip params.
  double xfxp(double x, double Q, double P2, int ip, int fl);

  /// Nuclear PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a a param for atomic mass number.
  std::vector<double> xfxa(double x, double Q, double a);

  /// Nuclear PDF: fills primitive 13 element array pointed at by @a results with
  /// \f$ x f(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$;
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a a param for atomic mass number.
  void xfxa(double x, double Q, double a, double* results);

  /// Nuclear PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a a param for atomic mass number.
  double xfxa(double x, double Q, double a, int fl);

  /// MRST QED PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$;
  /// @arg 13 = \f$ \gamma \f$.
  ///
  /// NB. Note extra element in this set for MRST photon.
  std::vector<double> xfxphoton(double x, double Q);

  /// MRST QED PDF: fills primitive 14 element array pointed at by @a results with
  /// \f$ x f(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$;
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  /// @arg 13 = \f$ \gamma \f$.
  ///
  /// NB. Note extra element in this set for MRST photon.
  void xfxphoton(double x, double Q, double* results);

  /// MRST QED PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$;
  /// @arg 7 = \f$ \gamma \f$.
  ///
  /// NB. Note extra element in this set for MRST photon.
  double xfxphoton(double x, double Q, int fl);

  /// Number of subsets available in the current distribution.
  int numberPDF();

  /// \f$ \alpha_\mathrm{s} \f$ used by the current PDF.
  double alphasPDF(double Q);

  /// Get order at which the PDF was fitted.
  int getOrderPDF();

  /// Perturbative order of parton evolution and \f$ \alpha_\mathrm{s} \f$ respectively.
  int getOrderAlphaS();

  /// Quark mass used for flavour @a f.
  double getQMass(int f);

  /// Threshold for flavour @a f.
  double getThreshold(int f);

  /// Number of flavours used in the current PDF set.
  int getNf();

  /// Value of QCD \f$ \lambda_4 \f$ for member @a m.
  double getLam4(int m);

  /// Value of QCD \f$ \lambda_5 \f$ for member @a m.
  double getLam5(int m);

  /// Minimum \f$ x \f$ value considered valid for this set, as specified by the set authors.
  double getXmin(int m);

  /// Maximum \f$ x \f$ value considered valid for this set, as specified by the set authors.
  double getXmax(int m);

  /// Minimum \f$ Q^2 \f$ value considered valid for this set, as specified by the set authors.
  double getQ2min(int m);

  /// Maximum \f$ Q^2 \f$ value considered valid for this set, as specified by the set authors.
  double getQ2max(int m);
  //@}



  /// @name Multi-set functions
  /// Additional functions for when more than 1 PDF set is being stored in memory.
  //@{

  /// The PDF set by file path, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSetM(int nset, const std::string& path);

  /// The PDF set by name and type, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSetByNameM(int nset, const std::string& name, SetType type);

  /// The PDF set by filename, see subdir @c PDFset of LHAPDF for choices.
  void initPDFSetByNameM(int nset, const std::string& filename);

  /// The choice of PDF subset out of one distribution.
  void initPDFM(int nset, int memset);

  /// Typical initializer (when multiple PDF sets need to be initialized) 
  /// with PDF set @a name and subset @a memset.
  /// @param nset Specifies the reference number for the set to be initialized.
  /// @param name Name of the desired set.
  /// @param type The type of PDF set (grid or data) by enum.
  /// @param memset PDF number within set @a name.
  /// Equivalent to @c initPDFSetByNameM + @c initPDFM.
  void initPDFByNameM(int nset, const std::string& name, SetType type, int memset);

  /// Typical initializer (when multiple PDF sets need to be initialized) 
  /// with PDF set @a name and subset @a memset.
  /// @param nset Specifies the reference number for the set to be initialized.
  /// @param filename Name of the grid or data file of the desired set.
  /// @param memset PDF number within set @a name.
  /// Equivalent to @c initPDFSetByNameM + @c initPDFM.
  void initPDFByNameM(int nset, const std::string& filename, int memset);

  /// Prints a brief description of the current PDF set to stdout.
  void getDescriptionM(int nset);

  /// Nucleon PDF: returns a vector @c x f_i(x, Q) with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  std::vector<double> xfxM(int nset, double x, double Q);

  /// Nucleon PDF: returns @c x f(x, Q) for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  double xfxM(int nset, double x, double Q, int fl);

  /// Photon PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a P2 and @a ip params.
  std::vector<double> xfxpM(int nset, double x, double Q, double P2, int ip);

  /// Photon PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a P2 and @a ip params.
  double xfxpM(int nset, double x, double Q, double P2, int ip, int fl);

  /// Nuclear PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a a param for atomic mass number.
  std::vector<double> xfxaM(int nset, double x, double Q, double a);

  /// Nuclear PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$.
  ///
  /// NB. Extra @a a param for atomic mass number.
  double xfxaM(int nset, double x, double Q, double a, int fl);

  /// MRST QED PDF: returns a vector \f$ x f_i(x, Q) \f$ with index \f$ 0 < i < 12 \f$.
  /// @arg 0..5 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$; 
  /// @arg 6 = \f$ g \f$; 
  /// @arg 7..12 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$;
  /// @arg 13 = \f$ \gamma \f$.
  std::vector<double> xfxphotonM(int nset, double x, double Q);

  /// MRST QED PDF: returns \f$ x f(x, Q) \f$ for flavour @a fl - this time the flavour encoding
  /// is as in the LHAPDF manual.
  /// @arg -6..-1 = \f$ \bar{t} \f$, ..., \f$ \bar{u} \f$, \f$ \bar{d} \f$;
  /// @arg 0 = \f$ g \f$
  /// @arg 1..6 = \f$ d \f$, \f$ u \f$, ..., \f$ t \f$;
  /// @arg 7 = \f$ \gamma \f$.
  double xfxphotonM(int nset, double x, double Q, int fl);

  /// Number of subsets available in the current distribution.
  int numberPDFM(int nset);

  /// \f$ \alpha_\mathrm{s} \f$ used by the current PDF.
  double alphasPDFM(int nset, double Q);

  /// Get order at which the PDF was fitted.
  int getOrderPDFM(int nset);

  /// Perturbative order of parton evolution and \f$ \alpha_\mathrm{s} \f$ respectively.
  int getOrderAlphaSM(int nset);

  /// Quark mass used for flavour @a f.
  double getQMassM(int nset, int f);

  /// Threshold for flavour @a f.
  double getThresholdM(int nset, int f);

  /// Number of flavours used in the current PDF set.
  int getNfM(int nset);

  /// Value of QCD \f$ \lambda_4 \f$ for member @a m.
  double getLam4M(int nset, int m);

  /// Value of QCD \f$ \lambda_5 \f$ for member @a m. 
  double getLam5M(int nset, int m);

  /// Minimum \f$ x \f$ value considered valid for this set, as specified by the set authors.
  double getXminM(int nset, int m);

  /// Maximum \f$ x \f$ value considered valid for this set, as specified by the set authors.
  double getXmaxM(int nset, int m);

  /// Minimum \f$ Q^2 \f$ value considered valid for this set, as specified by the set authors.
  double getQ2minM(int nset, int m);

  /// Maximum \f$ Q^2 \f$ value considered valid for this set, as specified by the set authors.
  double getQ2maxM(int nset, int m);
  //@}

  /// @name Misc functions
  //@{

  /// Choose level of noisiness.
  void setVerbosity(Verbosity noiselevel);

  /// Extrapolate beyond grid edges.
  void extrapolate(bool extrapolate=true);

  /// Set the LHAPATH variable (the location of the PDF sets directory).
  void setPDFPath(const std::string& path);

  //@}

};

#endif
